unit snksys;
{
SNK Shell Routines Library, v. 1.0 (Nov 20, 2001; Initial release)
v. 2.0 (April, 27, 2002). WinExec's, files, etc.
http://www.snkey.net
}
interface
uses Windows, Messages, SysUtils, Classes, Registry, ShlObj, ActiveX, ComObj,
 Forms, Graphics, ShellApi;

type
  TWindowOpt = class (TObject)
   private
    Left: integer;
    Top: integer;
    Height: integer;
    Width: integer;
    Max: boolean;
   public
    Path: string;
    RegKey: string;
    exti: integer;
    procedure ApplySize(frm: TForm);
    procedure SaveForm(frm: TForm; ps: string; const i: integer=-2);
    function LoadForm(frm: TForm):boolean;
    constructor Create;
  end;

procedure CreateShortcut(Target,Args,WorkDir,ShortCutName: String);

function GetDesktopDir:string;

function GetProgramsDir:string; //%programfiles%

function GetWinDir:string; //%SystemRoot%

function GetSysDir:string; //%SystemRoot%\System32

function GetAppIcon(FilePath: string; big: boolean): TIcon;

function ExecHelpContext(s,fn: string):boolean;

function ExecFile(app, param: string):boolean;

function ShellExecFile(fn: string):boolean;

function ShellExecFileEx(fn, dir, param: string):boolean;

function GetFileSz(fn: string; const scale:integer=0):double;

function FormatFileSize(d: double; const addbrake: boolean=false):string; //in bytes

function CorrectDate(d1:TDateTime):TDateTime; //Y2K

function IncYear(ADate: TDateTime; Years: Integer): TDateTime;

function CheckForBinary(fn: string): boolean; //if binary then true 

const
 KILOBYTE=1024;
 MEGABYTE=1024*1024;
 GIGABYTE=1204*1024*1024;

var
 { string resources}
 E_FILE_NOT_FOUND:string='File not found';
 E_SYS_ERROR:string='System error';
 E_NO_ACCESS:string='Access error: file is used by another application';
 E_OUT_OF_RES:string='Out of system resources';
 E_PATH_NOT_FOUND:string='Path not found';

implementation

uses SNKstr;

function GetAppIcon(FilePath: string; big: boolean): TIcon;
var
 ico: TIcon;
 h1,h2: HIcon;
begin
 Result:=nil;
 if not FileExists(FilePath) then exit;
 ico:=TIcon.Create;
 ico.Handle:=ExtractIconEx(PChar(FilePath),0,h1,h2,1);
 if big then begin
   ico.Handle:=h1;
   DestroyIcon(h2);
 end else begin
   ico.Handle:=h2;
   DestroyIcon(h1);
 end;
 Result:=ico;
end;

constructor TWindowOpt.Create;
begin
 inherited Create;
 Left:=1;
 Top:=1;
 Height:=0;//screen.Height-40;
 Width:=0;//screen.Width-40;
 Max:=false;
 exti:=-2;
end;

procedure TWindowOpt.SaveForm(frm: TForm; ps: string; const i: integer=-2);
var
 Reg: TRegistry;
begin
 Reg:=TRegistry.Create;
 Reg.RootKey:=HKEY_CURRENT_USER;
 Reg.OpenKey(AddSlash(RegKey)+frm.Name,true);
 Max:=(frm.WindowState in [wsMaximized]);
 Reg.WriteBool('Max',Max);
 if not Max then begin
   Reg.WriteInteger('Left',frm.Left);
   Reg.WriteInteger('Top',frm.Top);
   Reg.WriteInteger('Height',frm.Height);
   Reg.WriteInteger('Width',frm.Width);
 end;
 if (ps<>'') and (pos(ps,'A:')=0) and (pos(ps,'B:')=0) and (pos(ps,'a:')=0) and (pos(ps,'b:')=0)
   then Reg.WriteString('Path',ps);
 if (i>-2) then Reg.WriteInteger('ext',i);
 Reg.Free;
end;

function TWindowOpt.LoadForm(frm: TForm):boolean;
var
 Reg: TRegistry;
begin
 Result:=false;
 Reg:=TRegistry.Create;
 Reg.RootKey:=HKEY_CURRENT_USER;
 if Reg.OpenKey(AddSlash(RegKey)+frm.Name,false) then try
   Max:=Reg.ReadBool('Max');
   Left:=Reg.ReadInteger('Left');
   Top:=Reg.ReadInteger('Top');
   Height:=Reg.ReadInteger('Height');
   Width:=Reg.ReadInteger('Width');
   Path:=Reg.ReadString('Path');
   Result:=true;
  except
   on exception do ;
 end;
 try
   exti:=Reg.ReadInteger('ext');
  except
   on exception do exti:=-2;
 end;
 Reg.Free;
end;

procedure TWindowOpt.ApplySize(frm: TForm);
begin
 frm.Left:=Left;
 frm.Top:=Top;
 if Height>0 then frm.Height:=Height;
 if Width>0 then frm.Width:=Width;
 if Max then frm.WindowState:=wsMaximized;
end;

function GetDesktopDir:string;
var
 Reg: TRegistry;
begin
 Reg:=TRegistry.Create;
 Reg.RootKey:=HKEY_CURRENT_USER;
 Reg.OpenKeyReadOnly('\Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders');
 if Reg.ValueExists('Desktop') then
   Result:=AddSlash(Reg.ReadString('Desktop')) else
     Result:=AddSlash(Reg.ReadString('Common Desktop')); //XP/2000
 Reg.Free;
end;

function GetProgramsDir:string;
var
 Reg: TRegistry;
begin
 Reg:=TRegistry.Create;
 Reg.RootKey:=HKEY_CURRENT_USER;
 Reg.OpenKeyReadOnly('\Software\Microsoft\Windows\CurrentVersion');
 if Reg.ValueExists('ProgramFilesDir') then
   Result:=AddSlash(Reg.ReadString('ProgramFilesDir'));
 Reg.Free;
end;

function GetWinDir:string;
var
 td: array[0..255] of char;
begin
 GetWindowsDirectory(td,255);
 Result:=StrPas(td);
end;

function GetSysDir:string;
var
 td: array[0..255] of char;
begin
 GetSystemDirectory(td,255);
 Result:=StrPas(td);
end;

procedure CreateShortcut(Target,Args,WorkDir,ShortCutName: String);
var
 IObj    : IUnknown;
 Link    : IShellLink;
 IPFile  : IPersistFile;
 TargetW : WideString;
begin
  IObj := CreateComObject(CLSID_ShellLink);
  Link := IObj as IShellLink;
  IPFile  := IObj as IPersistFile;
  with Link do begin
    SetPath(PChar(Target));
    SetArguments(PChar(Args));
    SetShowCmd(SW_SHOWMINIMIZED);
    SetWorkingDirectory(PChar(WorkDir));
  end;
  TargetW := ShortCutName;
  IPFile.Save(PWChar(TargetW),False);
end;

function ExecHelpContext(s,fn: string):boolean;
var
 ss: PChar;
 i: integer;
 pi: ^integer;
begin
 ss:=PChar(s);
 pi:=Addr(ss);
 i:=pi^;
 Result:=winhelp(Application.Handle,PChar(fn),HELP_KEY,i);
end;

function ShellExecFile(fn: string):boolean;
var
 Abp: array[0..255] of Char;
 e: integer;
 s: string;
begin
 StrPCopy(Abp,fn);
 e:=ShellExecute(Application.Handle, 'open', Abp, nil, 'C:\', SW_SHOW);
 s:='';
 case e of
  ERROR_FILE_NOT_FOUND:s:=E_FILE_NOT_FOUND;
  ERROR_BAD_FORMAT:s:=E_SYS_ERROR;
  SE_ERR_ACCESSDENIED,SE_ERR_SHARE:s:=E_NO_ACCESS;
 end;
 if s<>'' then MessageBox(0,PChar(s),'Error',MB_OK);
 if e<32 then Result:=false else Result:=true;
end;

function ShellExecFileEx(fn, dir, param: string):boolean;
var
 Afn, Adir, Aparam: array[0..255] of Char;
 e: integer;
 s: string;
begin
 StrPCopy(Afn,fn);
 StrPCopy(Adir,dir);
 StrPCopy(Aparam,param);
 e:=ShellExecute(Application.Handle, 'open', Afn, Aparam, Adir, SW_SHOW);
 s:='';
 case e of
  ERROR_FILE_NOT_FOUND:s:=E_FILE_NOT_FOUND;
  ERROR_BAD_FORMAT:s:=E_SYS_ERROR;
  SE_ERR_ACCESSDENIED,SE_ERR_SHARE:s:=E_NO_ACCESS;
 end;
 if s<>'' then MessageBox(0,PChar(s),'Error',MB_OK);
 if e<32 then Result:=false else Result:=true;
end;

function ExecFile(app, param: string):boolean;
var
 pstr: Array[0..255] of Char;
 e: integer;
 s: string;
begin
 Result:=true;
 StrPCopy(pstr, app+' "'+param+'"');
 e:=winexec(pstr,SW_RESTORE);
 //e:=ShellExecute(Application.Handle, 'open', Abp, Afn, 'C:\', 0);
 if e>32 then exit;
 s:=E_SYS_ERROR;
 case e of
   0: s:=E_OUT_OF_RES;
   ERROR_FILE_NOT_FOUND: s:=E_FILE_NOT_FOUND;
   ERROR_PATH_NOT_FOUND: s:=E_PATH_NOT_FOUND;
   SE_ERR_SHARE: s:=E_NO_ACCESS;
 end;
 MessageBox(0,PChar(s),'Error',MB_OK);
 Result:=false;
end;

function GetFileSz(fn: string;const scale:integer=0):double;
var
 FileHandle: integer;
begin
 FileHandle := FileOpen(fn, fmOpenRead);
 if FileHandle > 0 then
   Result:=GetFileSize(FileHandle,nil)
 else
   Result:=0;
 FileClose(FileHandle);
 if scale>0 then Result:=Result / 1024; //kb
 if scale>1 then Result:=Result / 1024; //Mb
 if scale>2 then Result:=Result / 1024; //Gb
end;

function FormatFileSize(d: double; const addbrake: boolean=false):string;
var
 s: string;
begin
 if d<KILOBYTE then exit;
 if d>=KILOBYTE then begin
   if not addbrake then s:=' (' else s:='';
   s:=s+FloatToStrF(d/KILOBYTE,ffGeneral,3,2);
   Delete(s,8,10);
   s:=s+'kb';
 end;
 if d>=MEGABYTE then begin
   if not addbrake then s:=' (' else s:='';
   s:=s+FloatToStrF(d/MEGABYTE,ffGeneral,3,2);
   Delete(s,8,10);
   s:=s+'Mb';
 end;
 if d>=GIGABYTE then begin
   if not addbrake then s:=' (' else s:='';
   s:=s+FloatToStrF(d/GIGABYTE,ffGeneral,3,2);
   Delete(s,8,10);
   s:=s+'Gb';
 end;
 if not addbrake then s:=s+')';
 Result:=s;
end;

function CorrectDate(d1:TDateTime):TDateTime;
var
 Year,Month,Day:word;
begin
 DecodeDate(d1, Year, Month, Day);
 if Year>2050 then d1:=IncYear(d1,-100);
 Result:=d1;
end;

function IncYear(ADate: TDateTime; Years: Integer): TDateTime;
var
  D, M, Y: Word;
  Year,Month,Day: Longint;
begin
  DecodeDate(ADate, Y, M, D);
  Year := Y; Month:=M; Day:=D;
  Inc(Year, Years);
  Result := EncodeDate(Year, Month, Day) + Frac(ADate);
end;

function CheckForBinary(fn: string): boolean;
var
 f: file;
 Buf: array[1..2048] of Char;
 Rez,i: integer;
begin
 Result:=false;
 Assign(f,fn);
 FileMode := 0;
 try
   Reset(f,1);
   BlockRead(f, Buf, SizeOf(Buf),Rez);
  except
   on exception do begin
     CloseFile(f);
     exit;
   end;
 end;
 CloseFile(f);
 for i:=1 to Rez-1 do begin
   if Buf[i] in [#0..#8,#11,#12,#14..#19] then begin
     Result:=true;
     break;
   end;
 end;
end;

end.
